:
# NAME:
#	dns_convert.sh - convert DNS setup 
#
# SYNOPSIS:
#	dns_convert.sh [secondary]
#
# DESCRIPTION:
#	This script does everything.  It reads /etc/named.{boot,conf} and
#	creates a new named.{boot,conf} in the current directory which
#	is assumed to be a temporary directory prior to importing the
#	result into CVS. 
#
#	Without the 'secondary' argument, it reads /etc/named.{boot,conf} and 
#	for each primary zone, it creates the datafile that the .soa
#	file will expect to $INCLUDE (see
#	http://www.crufty.net/help/dns/dnsmagic.html)
#	and copy the data from the old zone file into it while removing the
#	SOA record and adding an RCSid.
#
#	It then runs dnsdeps -N named.{boot,conf} to see what files still
#	need to be copied from the old DNS tree to the new, and
#	fetches them.  This step is repeated until no files are
#	missing or it becomes obvious something is wrong.
#	
#	At the end of the exercise, you should have a new named.{boot,conf}
#	plus populated hosts and rev directories ready for import into
#	CVS.
#
#	You may need to edit .updsoarc (or .new_soarc) so that your .soa 
#	files will have the correct nameserver and mail address
#	in them, and if need be a .upd_sndbootrc if you need to force
#	an address for your nameserver into the named.boot.sec file.
#	For BIND-8, the relevant file is .slavezonesrc (again see
#	http://www.crufty.net/help/dns/dnsmagic.html for details.)
#	
#	The 'secondary' invocation is intended for running after the
#	new DNS data has been imported into CVS and checked out under
#	/var/named and will re-read /etc/named.{boot,conf}, and copy each
#	secondary zone file from its old location to the new (renaming
#	it as necessary). 
#
#	To cut over to the new setup, just make /etc/named.{boot,conf}
#	a symlink to /var/named/named.{boot,conf} and restart named.
#
# BUGS:
#	Compilcated by the need to support both BIND-4 and BIND-8
#	which have very different configuration files.
#	Not all features of modern named.boot files are converted to
#	named.conf but are included as comments.
#
#	We use /etc/named.conf if both it and /etc/named.boot exist.
#	Converting a BIND-8 named.conf is handled by first generating
#	an equivalent (more or less) named.boot file.
#
#	BIND-9 needs a controls section for rndc(8), we don't yet
#	handle this - use rndc-confgen(8).
#
# SEE ALSO:
#	cvs(1), dns_convzone.sh(8), dnsdeps.pl(8).
#		
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: dns_convert.sh,v 1.21 2002/06/07 19:31:16 sjg Exp $
#
#	@(#)Copyright (c) 1997-2001 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#

NDBOOT=${NDBOOT:-$DESTDIR/etc/named.conf}
NDBOOT=${NDBOOT:-$DESTDIR/etc/named.boot}
test -s $NDBOOT || exit 0

Mydir=`dirname $0`

case $NDBOOT in
*named*.conf)
	# we handle BIND-8, by creating a named.boot file
	BINDVERS=${BINDVERS:-8}
	sh $Mydir/ndconv.sh $NDBOOT > ndb.$$
	NDBOOT=ndb.$$
	OUTFILE=${OUTFILE:-named.conf}
	;;
esac

case "$1" in
sec*)	OUTFILE=/dev/null
	SECONDARY=
	PRIMARY=:
	;;
*)
	OUTFILE=${OUTFILE:-`basename $NDBOOT`}
	SECONDARY=:
	PRIMARY=
	;;
esac

# Does _anyone_ still use 4?
BINDVERS=${BINDVERS:-4}

case "$BINDVERS" in
4*)
	PZONES=/dev/null
	SZONES=/dev/null
	OPTS=/dev/null
	;;
*)
	PZONES=primary.zones
	SZONES=secondary.zones
	OPTS=opts.$$
        ;;
esac

# pick up the routines we need.
. $Mydir/dns_convzone.sh

# cleanup
trap 'rm -f .env .missing *.$$' 0

case "$1" in
sec*)
	conv_zones < $NDBOOT > /dev/null
	exit 0
	;;
esac

for f in makefile GNUmakefile .cvsignore
do
	test -s $Mydir/../$f && cp $Mydir/../$f .
done
test -s .cvsignore || cat <<! > .cvsignore
.depend
.bounce_dns
*.pid
named_dump.db
.nocvs
*.soa
!

cp .cvsignore hosts
cp .cvsignore rev

> .env
> $OUTFILE
	
case "$BINDVERS" in
4*)	;;
*)
	for f in $PZONES $SZONES
	do
		mkhead > $f
	done
	cat <<!EOH! > $OPTS
	directory "${DNSDIR:-/var/named}";
	// we keep the pid file local to ensure running
	// multiple nameservers does not cause problems.
	pid-file "named.pid";
	
	// uncomment query-source if you need to traverse a firewall...
	// query-source address * port 53;

!EOH!
	;;
esac

conv_zones < $NDBOOT >> $OUTFILE

case "$OUTFILE" in
*named*.conf)
	mv $OUTFILE $OUTFILE.$$
	(
	mkhead 
	echo
	echo "options {"
	cat $OPTS
	echo "};"
	echo
	case "$BINDVERS" in
	9*)
		cat <<'!EOL!'
logging {
	// log what we can		  
	category unmatched { default_syslog; };
	category default { default_syslog; };
};
!EOL!
		;;
	*)
		cat <<'!EOL!'
logging {
	// we want to know about all problems
	// so that the regression suite will pick them up
	// we only need this on the master.
	category cname { default_syslog; };
	category lame-servers { default_syslog; };
	category insist { default_syslog; };
	// we may also want some of these
	category xfer-out { default_syslog; };
	category statistics { default_syslog; };
	category update { default_syslog; };
	category security { default_syslog; };
	category os { default_syslog; };
	category notify { default_syslog; };
	category response-checks { default_syslog; };
	category maintenance { default_syslog; };
};

!EOL!
		;;
	esac
	cat $OUTFILE.$$
	echo
	echo 'include "primary.zones";'
	echo 'include "secondary.zones";'
	) | sed 's,//;;*,// ,' > $OUTFILE
	;;
esac

# now we want to find out any missing files...
case "$PRIMARY" in
"")
	. ./.env
	here=`pwd`
	# this might take a few iterations
	# but we stop when nothing is happening.
	touch .missing.$$
	if [ -s $Mydir/dnsdeps.sh ]; then
		DNSDEPS="sh $Mydir/dnsdeps.sh"
	else
		DNSDEPS="perl $Mydir/dnsdeps.pl"
	fi
        
	for i in 1 2 3 4 5 6 7 8 9
	do
		eval $DNSDEPS -N ${OUTFILE} 2>&1 |
		grep cannot | sort -u | awk '{ print $NF }' > .missing
		test -s .missing || break
		cmp -s .missing .missing.$$ && break
		(cd $dir && cat $here/.missing | cpio -pdm $here)
		mv .missing .missing.$$
	done
	if test -s .missing; then
		echo Warning: after $i iterations, cannot locate:
		cat .missing
	fi
	;;
esac


