# NAME:
#	strftime - format date string
#
# SYNOPSIS:
#	$str = &strftime($fmt, $time);
#
#	$str = &ctime($time);
#
# DESCRIPTION:
#	Not quite the ANSI C strftime(), but close enough to be
#	quite useful.  The table below indicates the substitution
#	strings used.  ctime() is implemented via strftime().
#.nf
#
#	%%	Same as %
#	%a	Abbreviated weekday name
#	%A	Full weekday name
#	%b	Abbreviated month name
#	%B	Full month name
#	%c	Local date and time representation
#	%C	Date and time representation as produced by date(1) 
#	%d	Day of month (01-31)
# 	%D	Date as %m/%d/%y
#	%e	Day of month (1-31; single digits are preceded
#		by a blank) 
# 	%h	Abbreviated month name.
#	%H	Hour (00-23)
# 	%I	Hour (01-12)
# 	%i	Hour (1-12; single digits are preceded by a blank) 
#	%j	Day number of year (001-366)
# 	%m	Month number (01-12)
#	%M	Minute (00-59)
# 	%n	Same as \n
#	%p	Local equivalent of either AM or PM
#	%r	Time as %I:%M:%S [AM|PM]
#	%R	Time as %H:%M
# 	%s	Seconds since the epoch
# 	%S	Seconds (00-61), allows for leap seconds
#	%t	Insert a tab
#	%T	Time as %H:%M:%S
# 	%U	Week number of year (00-53), Sunday is the
#		first day of week 1
#	%w	Weekday number (0-6), Sunday = 0
#	%W	Week number of year (00-53), Monday is the
# 		first day of week 1
#	%x	Local date representation
# 	%X	Local time representation
#	%y	Year within century (00-99)
# 	%Y	Year as ccyy (for example, 1986)
#	%Z	time zone name or no characters if no time zone
# 		exists 
#.nf
#
#	Most translations are setup regardless as this is
#	cheaper than determining whether they are needed.
#	Note that %e and %i are probably non-standard, but quite
#	useful. 
#
# RETURNS:
#	A formatted string.
#
# BUGS:
#	BSD systems do not use TZ in the environment the same
#	way as System V does.  The mechanism used here to
#	determin the Time Zone name on BSD systems is an ugly
#	hack.
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>

# RCSid:
#	$Id: strftime.pl,v 1.9 2000/07/02 08:56:18 sjg Exp $
#
#	@(#)Copyright (c) 1991, Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

package strftime;

sub main'ctime {
  local($time) = @_;

  &main'strftime('%a %b %e %T %Y', $time);
}

sub main'strftime {
  local($fmt,$time) = @_;
  local($mday, $mon, $isdst,$junk);
  local($a,$A,$b,$B,$c,$C,$d,$D);
  local($e,$h,$H,$I,$i,$j,$m,$M,$p);
  local($r,$R,$s,$S,$T,$U,$w,$W,$x,$X,$y,$Y,$Z);

  $time = time unless ($time > 0);
  &init_sftime unless (defined(@Months)); # 1st time initialization
  $s = $time;
  
  ($S, $M, $H, $d, $mon, $y, $w, $j, $isdst) =
    ($TZ eq 'GMT') ? gmtime($time) : localtime($time);

  # set up some simple ones.
  $m = $mon + 1;
  $Y = $y + 1900;
  $A = $Days[$w];
  $B = $Months[$mon];

  # now some that we should avoid if not needed
  if ($fmt =~ m/%[aC]/) {
    ($a,$junk) = unpack("A3A*", $A);
  }
  if ($fmt =~ m/%[bhC]/) {
    ($b,$junk) = unpack("A3A*", $B);
    $h = $b;
  }
  if ($fmt =~ m/%[IXprc]/) {
    $p = "AM";
    $I = $H;
    if ($I > 12) {
      $p = "PM";
      $I -= 12;
    } elsif ($I == 12) {
      $p = "PM";
    } elsif ($I == 0) {
      $I = 12;
    }
  }
  if ($fmt =~ m/\%[CZ]/) {
    if ($TZ eq '') {
      # we are probably a BSD system.
      $TZ = `date`;	# yuk!
      $TZ =~ s/.* ([A-Z]+) ${Y}/$1/;
      $TZ =~ s/\n//;
    }
    $Z = $TZ;
    # this incantation is lifted from Waldemar Kebsch's ctime.pl
    # it helps extract the correct TZ name from $TZ.
    if($Z=~/^([^:\d+\-,]{3,})([+-]?\d{1,2}(:\d{1,2}){0,2})([^\d+\-,]{3,})?/){
      $Z = $isdst ? $4 : $1;
    }
  }
  if ($fmt =~ m/%U/) {
    $U = &week_num($j, $w, 0);
  }
  if ($fmt =~ m/%W/) {
    $W = &week_num($j, $w, 1);
  }

  # format these correctly
  $e = ($d < 10) ? " $d" : $d;	# non-standard ?
  $d = ($d < 10) ? "0$d" : $d;
  $m = ($m < 10) ? "0$m" : $m;
  $y = ($y < 10) ? "0$y" : $y;
  $H = ($H < 10) ? "0$H" : $H;
  $i = ($I < 10) ? " $I" : $I;	# non-standard
  $I = ($I < 10) ? "0$I" : $I;
  $M = ($M < 10) ? "0$M" : $M;
  $S = ($S < 10) ? "0$S" : $S;
  $j = ($j < 100) ? ($j < 10) ? "00$j" : "0$j" : $j;

  # deal with y>100
  $y = $Y;
  $y =~ s/.*(..)$/$1/;

  # more simple assignments that depend on the above
  $r = "$I:$M:$S $p";
  $R = "$H:$M";
  $T = "$R:$S";
  $D = "$m/$d/$y";
  $x = "$e/$m/$Y";
  $X = "$i:$M $p";
  $c = "$x $X";
  $C = "$a $b $e $T $Z $Y";	# same as date(1)
  
  # finaly get ready to format the desired string.
  
  $fmt =~ s/%([^%])/\${$1}/g;	# make %a ${a} etc.
  $fmt =~ s/%%/\\%/g;		# handle a '%' in the desired string.

  eval "\"$fmt\"";		# eval and return string.
}

# calculate a week number

sub week_num {
  local($yday, $wday, $start) = @_;
  local($week);

  $week = int($yday / 7) + 1;
  if (($yday % 7) <= ($wday - $start)) {
    $week--;
  }
  $week;
}

sub init_sftime {
  @Months = ('January', 'February', 'March', 
	     'April', 'May', 'June', 
	     'July', 'August', 'September',
	     'October', 'November', 'December' );

  @Days = ('Sunday', 'Monday', 'Tuesday', 'Wednesday',
	   'Thursday', 'Friday', 'Saturday' );

  # TZ is only used on System V machines.
  # setting TZ is definitely not desired under SunOS 4.0.2
  # other BSD systems probably feel the same.
  if (!defined($TZ)) {
    $TZ = defined($ENV{'TZ'}) ? ( $ENV{'TZ'} ? $ENV{'TZ'} : 'GMT' ) : '';
  }

  $n="\n";
  $t="\t";
}

1;
